#include "olathread.h"


olaThread::olaThread(QObject *parent, int layers) :
    m_counter(0)
  , m_layers(layers)
{
    Q_UNUSED(parent);
    gettimeofday(&m_last_data, NULL);
    for (int i=0; i < MAX_LAYERS; i++)
    {
      for (int j=0; j < LAYER_CHANNELS; j++)
      {
        m_dmx[i][j] = 0;
      }
    }
    init();
}

olaThread::~olaThread() {
    stop();
}

void olaThread::init()
{
    m_clientWrapper = new ola::client::OlaClientWrapper;
    Q_CHECK_PTR(m_clientWrapper);
    if (ola::InitLogging(ola::OLA_LOG_INFO , ola::OLA_LOG_STDERR)) {
        qDebug() << "ola logging debug working";
    } else {
        qWarning() << "Can not init ola logging";
    }
    if (!m_clientWrapper->Setup()) {
        qCritical("olaThread::olaStart| Failed Setup() in Client Wrapper");
        exit(EXIT_FAILURE);
    }
    m_client = m_clientWrapper->GetClient();
    m_client->SetDMXCallback(ola::NewCallback(this, &olaThread::NewDmx));
    m_clientWrapper->GetSelectServer()->RegisterRepeatingTimeout(4000, ola::NewCallback(this, &olaThread::CheckDataLoss));
    m_client->SetCloseHandler(ola::NewSingleCallback(this, &olaThread::socketClosed));
}

void olaThread::run()
{
    m_clientWrapper->GetSelectServer()->Run();
}

void olaThread::stop()
{
    if (m_clientWrapper != NULL)
    {
        QSet<int> temp = Settings::getInstance()->getUniverses();
        foreach (const int &i, temp) {
            m_client->RegisterUniverse(i, ola::client::UNREGISTER, ola::NewSingleCallback(this, &olaThread::RegisterComplete));
        }
        m_clientWrapper->GetSelectServer()->Terminate();
        m_clientWrapper = NULL;
        m_client = NULL;
    }
}

void olaThread::NewDmx(const ola::client::DMXMetadata &data,
                       const ola::DmxBuffer &buffer)
{
    m_counter++;
    gettimeofday(&m_last_data, NULL);
    foreach (const dmxSetting &i, Settings::getInstance()->getDmxSettings()) {
      if(i.universe == data.universe && i.address > -1) {
        for (int j = 0; j < LAYER_CHANNELS; j++){
          int value = buffer.Get((i.address) + j);
          if (m_dmx[i.layer][j] != value) {
            m_dmx[i.layer][j] = value;
            emit dmxOutput(i.layer,j,value);
          }
        }
      }
    }
}

/**
 * Check for data loss each 4 seconds.
 */
bool olaThread::CheckDataLoss() {
  struct timeval now, diff;
  if (timerisset(&m_last_data)) {
    gettimeofday(&now, NULL);
    timersub(&now, &m_last_data, &diff);
    if (diff.tv_sec > 4 || (diff.tv_sec == 4 && diff.tv_usec > 4000000)) {
      qInfo()<< "olaThread| Can not read one or several universes";
    }
  }
  return true;
}

void olaThread::resendDmx()
{
    for (int i = 0; i < m_layers; i++) {
        for (int j = 0; j < LAYER_CHANNELS; j++){
            emit dmxOutput(i, j, m_dmx[i][j]);
        }
    }
}

void olaThread::socketClosed()
{
    qWarning("ola closed connection. Try reopening it... ");
    m_clientWrapper->GetSelectServer()->Terminate();
    m_client = NULL;
    m_clientWrapper = NULL;
    init();
    registerUniverse();
    run();
}
